#!/usr/bin/env python
'''
 File    : framework.py
 Revision: $Revision$
 Author  : William McLendon, Narayan Desai
 Description:
        implementation of test driver framework.
        Contains the following classes:
                TestList
		TestSet
'''
import re
import sys
from md5 import md5
from string import join
from time import sleep
from xml.dom.minidom import parse,Document
from graph import DiGraph
from exceptions import Exception
from errorhandlers import TestSetupError,TestError
from testbase import Node,Msgbuf,TestInfo,dbWrite

# Import the new testHandler class 
# ADD CUSTOM TESTS TO gTestClassMap list {key=<element name>:<testHandler class> }
from testlibs import SSSTest,shellTest,tcpipTest,failTest,passTest
gTestClassMap = {'ssslibTest':SSSTest, 'shellTest':shellTest, 'tcpipTest':tcpipTest, \
                 'failTest':failTest, 'passTest':passTest}



# ------------------------------------------------------------------------------
# class TestList
# ------------------------------------------------------------------------------
class TestList(Node):
    ''' class TestList: high level object containing the DOM trees for all test bodies. '''
    __attrlist__=['numReps','iterDelay','preDelay','postDelay']

    testtypes=[]
    typemap=gTestClassMap

    for k in gTestClassMap.keys():
        testtypes.append(k)

    def __setdefaults__(self):
        self.options   = {}
        self.numReps   = '1'
        self.iterDelay = '0.0'
        self.preDelay  = '0.0'
        self.postDelay = '0.0'

    def getDeps(self,i, child):
        ''' returns the list of dependencies as tuples (name,type,minPctMatch,maxPctMatch)'''
        deps = []
        if child.tagname=='dependency':
          deps.append( (child.name,child.type, child.minPctMatch, child.maxPctMatch) )
        if hasattr(child, 'children'):
          for c in child.children:
            deps += self.getDeps(i+1,c)
        return deps

    def prepare(self,options):
        '''TestList.prepare()'''
        self.options = options
        self.scratch = {}
        self.tests=[]
        # Sort out the tests into a proper execution order (Topological Sort)
        G = DiGraph()
        for child in self.children:
            G.add_vertex(child.name)
        for child in self.children:
            deps = self.getDeps(0,child)
            for i in deps:
              if self.options.has_key('debug'): print "D:\tAdding order dependency edge: %s->%s"%(child.name,i[0])
              G.add_edge(child.name, i[0])
        G.reverse()	# reverse the edges
        L = G.topological_sort()
        for t in L:
            for child in self.children:
                if (child.tagname in self.testtypes) and (child.name == t):
                    self.tests.append(child)

    def findTestByName(self, tname):
        for t in self.tests:
            if t.name == tname:
                return t
        return None

    def pcheck(self, itest):
        rval = [1,'']
        if self.options.has_key('debug'): print 'D:\tpcheck() <%s>'%(itest.tagname)
        if itest.tagname != 'dependency':
            if hasattr(itest,'children'):
                rlist=[]
                # append to rval[1] (expression)
                if itest.tagname in ['OR','NOR','AND','NAND']: rval[1] += '('
                elif itest.tagname == 'NOT' and len(itest.children)>1:
                    raise TestError, '<NOT> must have only one child node'
                for i in itest.children:
                    tval = self.pcheck(i)
                    rlist.append(tval[0])
                    rval[0] = tval[0]
                    rval[1] += tval[1]
                    for j in tval[2:]: 
                        rval.append(j)
                    if itest.tagname == 'OR' or itest.tagname=='NOR':
                        if i != itest.children[len(itest.children)-1]: 
                            rval[1] += ' '+itest.tagname+' '
                    elif itest.tagname == 'AND' or itest.tagname=='NAND':
                        if i != itest.children[len(itest.children)-1]:
                            rval[1] += ' '+itest.tagname+' '
                    elif itest.tagname == 'NOT':
                        rval[1] = ' '+itest.tagname+' '+rval[1]
                if itest.tagname in ['OR','NOR','AND','NAND']: rval[1] += ')'
                if itest.tagname=='OR' or itest.tagname=='NOR':
                    rval[0] = 0
                    for i in rlist:
                        if i == 1:
                            rval[0] = 1
                    if itest.tagname=='NOR': 
                        rval[0] = not rval[0]
                elif itest.tagname=='AND' or itest.tagname=='NAND':
                    rval[0] = 1
                    for i in rlist:
                        if i == 0:
                            rval[0] = 0
                    if itest.tagname=='NAND': 
                        rval[0] = not rval[0]
                elif itest.tagname=='NOT':
                    rval[0] = not rlist[0]
        elif itest.tagname=='dependency':
            rval = [1, itest.name]
            parent = self.findTestByName(itest.name)
            parentPctMatch = parent.pctMatched
            rval.append((parent.name,parent.pctMatched,itest.minPctMatch,itest.maxPctMatch))
            if parent.pctMatched < float(itest.minPctMatch) or \
                    parent.pctMatched > float(itest.maxPctMatch):
                rval[0] = 0
        return rval


    def execute(self):
        '''TestList.execute() : Main driver for tests. '''

        # print column headers
        print ""
        print "iterations\ttest name\t% matched\tPass/Fail\tmessage"
        print "----------\t---------\t---------\t---------\t----------"

        sleep(float(self.preDelay))
     
        for listIter in range(int(self.numReps)):

            # Iterate over all tests in the sequence
            for test in self.tests:
                if self.options.has_key('verbose'): print "V:\tExecuting test '%s'"%(test.name)
                # Check the prerequisites
                try:
                    prereqs = self.pcheck(test)
                except TestError,msg:
                    print "Fatal error: %s"%(msg)
                    sys.exit(1)
                if prereqs[0] == 1:
                    try:
                        test.prepare(self.options)
                    except TestSetupError, e:
                        print "Test %s setup failed because of error: %s"%(test.name,e)
                    while test.irep < test.numReps:
                        try:
                            test.runTests(self.scratch)
                        except TestSetupError,e:
                            if self.options.has_key('verbose'):
                                print "V:\t[%d of %d] in %s failed execution :%s:"% \
                                        (test.irep,test.numReps,test.name,e)
                            if test.onMismatch == "BREAK": 
                                test.printSummary("BREAK")
                                break
                            elif test.onMismatch == "HALT": 
                                test.printSummary("HALT")
                                sys.exit(100)
                    if test.irep == test.numReps:
                        test.printSummary("")
                else:
                    print "[0 of %s]\t%-10s\tDEPENDENCY FAILURE(S)\t\t%s"%(test.numReps,test.name,prereqs[1])
                    for i in prereqs[2:]:
                        print "\t\t\t\t%s expected [%.1f%% : %.1f%%], got %s"%(i[0],float(i[2]),\
                          float(i[3]),i[1])
#                    for i in prereq[0][1]:
#                        print "\t\t\t\t%s expected [%.1f%% : %.1f%%], got %s"%(i[0],float(i[1]),float(i[2]),i[3])
            if listIter < int(self.numReps)-1:
                sleep(float(self.iterDelay)) 

        sleep(float(self.postDelay))


# ------------------------------------------------------------------------------
# class TestSet
# ------------------------------------------------------------------------------
class TestSet(Node):
    __textfields__=['name']
    typemap={'info':TestInfo,'testList':TestList} 

    def __setdefaults__(self):
        pass

    def execute(self, options):
        ''' this is the file that is called by main() '''
        self.options = options
        # Set the test file options
        for child in self.children:
            if child.tagname=='testList':
                child.prepare(self.options)
                child.execute()
                    

# EOF
